///File : MainPanel.cpp
//------------------------------------------------------------------------------
/*
    This file is part of resistancevalue.

    Resistancevalue is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Resistancevalue is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with resistancevalue.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainPanel.hpp"
#include "ResistanceColorDialog.hpp"
#include "findFile.hpp"
//------------------------------------------------------------------------------
BEGIN_EVENT_TABLE(PanelResistanceColorToValue, wxPanel)
    EVT_BUTTON(Id_Bitmap_Bt_Color_1, PanelResistanceColorToValue::OnBitmapBt1)
    EVT_BUTTON(Id_Bitmap_Bt_Color_2, PanelResistanceColorToValue::OnBitmapBt2)
    EVT_BUTTON(Id_Bitmap_Bt_Color_3, PanelResistanceColorToValue::OnBitmapBt3)
    EVT_BUTTON(Id_Bitmap_Bt_Color_4, PanelResistanceColorToValue::OnBitmapBt4)
END_EVENT_TABLE()
//------------------------------------------------------------------------------
void PanelResistanceColorToValue::OnBitmapBt1(wxCommandEvent &event)
{
    DialogBandColorChoice MyDialog(this, wxID_ANY, m_colorNumberBand[0]);
    if(MyDialog.ShowModal() == wxID_OK)
    {
        m_colorNumberBand[0] = MyDialog.GetColorNumber();
        drawBitmapButton();
    }
    event.Skip();
}
//------------------------------------------------------------------------------
void PanelResistanceColorToValue::OnBitmapBt2(wxCommandEvent &event)
{
    DialogBandColorChoice MyDialog(this, wxID_ANY, m_colorNumberBand[1]);
    if(MyDialog.ShowModal() == wxID_OK)
    {
        m_colorNumberBand[1] = MyDialog.GetColorNumber();
        drawBitmapButton();
    }
    event.Skip();
}
//------------------------------------------------------------------------------
void PanelResistanceColorToValue::OnBitmapBt3(wxCommandEvent &event)
{
    DialogBandColorChoice MyDialog(this, wxID_ANY, m_colorNumberBand[2]);
    if(MyDialog.ShowModal() == wxID_OK)
    {
        m_colorNumberBand[2] = MyDialog.GetColorNumber();
        drawBitmapButton();
    }
    event.Skip();
}
//------------------------------------------------------------------------------
void PanelResistanceColorToValue::OnBitmapBt4(wxCommandEvent &event)
{
    DialogBandColorChoiceBand4 MyDialog(this, wxID_ANY, m_colorNumberBand[3] - 10);
    if(MyDialog.ShowModal() == wxID_OK)
    {
        m_colorNumberBand[3] = MyDialog.GetColorNumber() + 10;
        drawBitmapButton();
    }
    event.Skip();
}
//------------------------------------------------------------------------------
void PanelResistanceColorToValue::drawBitmapButton()
{
    m_BitmapBtColor1->SetBitmapLabel(wxBitmap(m_ImagesForButtons[m_colorNumberBand[0]]));
    m_BitmapBtColor2->SetBitmapLabel(wxBitmap(m_ImagesForButtons[m_colorNumberBand[1]]));
    m_BitmapBtColor3->SetBitmapLabel(wxBitmap(m_ImagesForButtons[m_colorNumberBand[2]]));
    m_BitmapBtColor4->SetBitmapLabel(wxBitmap(m_ImagesForButtons[m_colorNumberBand[3]]));

    int percentageError = 0;
        if(m_colorNumberBand[3] == 10)
        percentageError = 5;
    else if(m_colorNumberBand[3] == 11)
        percentageError = 10;
    else if(m_colorNumberBand[3] == 12)
        percentageError = 20;

    double valeur = (m_colorNumberBand[0] * 10 + m_colorNumberBand[1]) * pow(10, m_colorNumberBand[2]);

    wxString Content;
    Content << _T("R = ") << m_colorNumberBand[0] << m_colorNumberBand[1] << _T(" * 10e") << m_colorNumberBand[2] << _T(" ±") << percentageError << _T("% Ω\n");
    Content << _T("R = ") << valeur << _T(" ±") << percentageError << _T("% Ω\n");
    Content << valeur - valeur * ((double)percentageError / 100) << _T(" Ω < R < ") << valeur + valeur * ((double)percentageError / 100) << _T(" Ω");

    m_TextInfoResistance->SetFont(wxFont(20, wxSWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL ));
    m_TextInfoResistance->SetValue(Content);
}
//------------------------------------------------------------------------------
void PanelResistanceColorToValue::initBitmap()
{
    wxColour ColorCodeResistance[12];
    ColorCodeResistance[0] = wxColour(0,0,0);
    ColorCodeResistance[1] = wxColour(184,134,11);
    ColorCodeResistance[2] = wxColour(255,0,0);
    ColorCodeResistance[3] = wxColour(255,165,0);
    ColorCodeResistance[4] = wxColour(255,255,0);
    ColorCodeResistance[5] = wxColour(154,205,50);
    ColorCodeResistance[6] = wxColour(100,149,237);
    ColorCodeResistance[7] = wxColour(238,130,238);
    ColorCodeResistance[8] = wxColour(160,160,160);
    ColorCodeResistance[9] = wxColour(255,255,255);
    ColorCodeResistance[10] = wxColour(255, 215, 0);
    ColorCodeResistance[11] = wxColour(192, 192, 192);

    for(int i = 0; i < 12; i++)
    {
        wxImage tempImg(200,60/*m_BitmapBtColor1->GetSize().GetWidth(),m_BitmapBtColor1->GetSize().GetHeight()*/);
        tempImg.SetRGB(wxRect(0,0,tempImg.GetWidth(), tempImg.GetHeight()), ColorCodeResistance[i].Red(), ColorCodeResistance[i].Green(), ColorCodeResistance[i].Blue());
        m_ImagesForButtons[i] = tempImg;
    }

    wxImage tempImg(50,60/*m_BitmapBtColor1->GetSize().GetWidth(),m_BitmapBtColor1->GetSize().GetHeight()*/);
    tempImg.SetRGB(wxRect(0,0,tempImg.GetWidth(), tempImg.GetHeight()), GetBackgroundColour().Red(), GetBackgroundColour().Green(), GetBackgroundColour().Blue());

    wxBitmap bmp(tempImg);
    wxMemoryDC dc;
    dc.SelectObject(bmp);

    //dc.DrawRectangle(wxRect(0, 0, dc.GetSize().GetWidth(), dc.GetSize().GetHeight()));
    dc.SetFont(wxFont(35, wxSWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL ));
    dc.DrawText(_T("Ø"), 5, 0);

    m_ImagesForButtons[12] = bmp.ConvertToImage();
}
//------------------------------------------------------------------------------
PanelResistanceColorToValue::PanelResistanceColorToValue( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    wxBoxSizer *MainSizer = new wxBoxSizer( wxVERTICAL );

    m_colorNumberBand[0] = 4;
    m_colorNumberBand[1] = 7;
    m_colorNumberBand[2] = 1;
    m_colorNumberBand[3] = 10;

    initBitmap();

	wxBoxSizer* SizerChoices = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer7 = new wxBoxSizer( wxVERTICAL );

	m_staticText1 = new wxStaticText( this, wxID_ANY, _("1st band :"));
	bSizer7->Add( m_staticText1, 0, wxALL, 5 );

	m_BitmapBtColor1 = new  wxBitmapButton( this, Id_Bitmap_Bt_Color_1, wxNullBitmap,  wxDefaultPosition, wxSize(-1, 60) );
	bSizer7->Add( m_BitmapBtColor1, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer7, 1, 0, 5 );

	wxBoxSizer* bSizer8;
	bSizer8 = new wxBoxSizer( wxVERTICAL );

	m_staticText2 = new wxStaticText( this, wxID_ANY, _("2nd band :"));
	bSizer8->Add( m_staticText2, 0, wxALL, 5 );

	m_BitmapBtColor2 = new  wxBitmapButton( this, Id_Bitmap_Bt_Color_2, wxNullBitmap,  wxDefaultPosition, wxSize(-1, 60) );
	bSizer8->Add( m_BitmapBtColor2, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer8, 1, 0, 5 );

	wxBoxSizer* bSizer9 = new wxBoxSizer( wxVERTICAL );

	m_staticText3 = new wxStaticText( this, wxID_ANY, _("3rd band (multiplier) :"));
	bSizer9->Add( m_staticText3, 0, wxALL, 5 );

	m_BitmapBtColor3 = new wxBitmapButton( this, Id_Bitmap_Bt_Color_3, wxNullBitmap,  wxDefaultPosition, wxSize(-1, 60) );
	bSizer9->Add( m_BitmapBtColor3, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer9, 1, 0, 5 );

	wxBoxSizer* bSizer10;
	bSizer10 = new wxBoxSizer( wxVERTICAL );

	m_staticText4 = new wxStaticText( this, wxID_ANY, _("4th band (tolerance) :"));
	bSizer10->Add( m_staticText4, 0, wxALL, 5 );

	m_BitmapBtColor4 = new wxBitmapButton( this, Id_Bitmap_Bt_Color_4, wxNullBitmap,  wxDefaultPosition, wxSize(-1, 60) );
	bSizer10->Add( m_BitmapBtColor4, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer10, 1, 0, 5 );

	MainSizer->Add(SizerChoices, 0, wxEXPAND);

	m_TextInfoResistance = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY );
	MainSizer->Add( m_TextInfoResistance, 1, wxALL|wxEXPAND, 5 );

	SetSizer(MainSizer);

	drawBitmapButton();
}
//------------------------------------------------------------------------------
BEGIN_EVENT_TABLE(PanelResistanceValueToColor, wxPanel)
    EVT_CHOICE(wxId_Choice,  PanelResistanceValueToColor::OnChoice)
END_EVENT_TABLE()
//------------------------------------------------------------------------------
void PanelResistanceValueToColor::OnChoice(wxCommandEvent& event)
{
    if(m_ChoiceBand1->GetCurrentSelection() != -1 && m_ChoiceBand2->GetCurrentSelection() != -1 && m_ChoiceBand3->GetCurrentSelection() != -1 && m_ChoiceBand4->GetCurrentSelection() != -1)
    {
        wxImage newImg(ImgResistance);

        wxColour ColorCodeResistance[10];
        ColorCodeResistance[0] = wxColour(0,0,0);
        ColorCodeResistance[1] = wxColour(184,134,11);
        ColorCodeResistance[2] = wxColour(255,0,0);
        ColorCodeResistance[3] = wxColour(255,165,0);
        ColorCodeResistance[4] = wxColour(255,255,0);
        ColorCodeResistance[5] = wxColour(154,205,50);
        ColorCodeResistance[6] = wxColour(100,149,237);
        ColorCodeResistance[7] = wxColour(238,130,238);
        ColorCodeResistance[8] = wxColour(160,160,160);
        ColorCodeResistance[9] = wxColour(255,255,255);

        int selection = m_ChoiceBand1->GetCurrentSelection();
        newImg.SetRGB(wxRect(163, 73, 30, 78), ColorCodeResistance[selection].Red(), ColorCodeResistance[selection].Green(), ColorCodeResistance[selection].Blue());
        selection = m_ChoiceBand2->GetCurrentSelection();
        newImg.SetRGB(wxRect(200, 73, 30, 78), ColorCodeResistance[selection].Red(), ColorCodeResistance[selection].Green(), ColorCodeResistance[selection].Blue());
        selection = m_ChoiceBand3->GetCurrentSelection();
        newImg.SetRGB(wxRect(250, 73, 30, 78), ColorCodeResistance[selection].Red(), ColorCodeResistance[selection].Green(), ColorCodeResistance[selection].Blue());

        selection = m_ChoiceBand4->GetCurrentSelection();
        if(selection <= 1) // Il y a une 3é bande seulement pour 10% et 5% (Soit 0 ou 1).
        {
            ColorCodeResistance[0] = wxColour(255, 215, 0);
            ColorCodeResistance[1] = wxColour(192, 192, 192);
            newImg.SetRGB(wxRect(300, 73, 20, 78), ColorCodeResistance[selection].Red(), ColorCodeResistance[selection].Green(), ColorCodeResistance[selection].Blue());
        }

        m_ImgResistance->SetBitmap(wxBitmap(newImg));
    }
}
//------------------------------------------------------------------------------
PanelResistanceValueToColor::PanelResistanceValueToColor( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    wxArrayString StringTblChoice1and2, StringTblChoice3, StringTblChoice4;
    for(int i = 0; i <= 9; i++)
    {
        wxString str;
        str << i;
        StringTblChoice1and2.Add(str);
        str = _T("");
        str << _T("10e") << i;
        StringTblChoice3.Add(str);
    }

    StringTblChoice4.Add(_T("±5%"));
    StringTblChoice4.Add(_T("±10%"));
    StringTblChoice4.Add(_T("±20%"));

	wxBoxSizer* MainSizer = new wxBoxSizer( wxVERTICAL );
	wxBoxSizer* SizerChoices = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer7 = new wxBoxSizer( wxVERTICAL );

	m_staticText1 = new wxStaticText( this, wxID_ANY, _("1st band :"));
	bSizer7->Add( m_staticText1, 0, wxALL, 5 );

	m_ChoiceBand1 = new wxChoice( this, wxId_Choice, wxDefaultPosition, wxDefaultSize, StringTblChoice1and2);
	bSizer7->Add( m_ChoiceBand1, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer7, 1, 0, 5 );

	wxBoxSizer* bSizer8;
	bSizer8 = new wxBoxSizer( wxVERTICAL );

	m_staticText2 = new wxStaticText( this, wxID_ANY, _("2nd band :"));
	bSizer8->Add( m_staticText2, 0, wxALL, 5 );

	m_ChoiceBand2 = new wxChoice( this, wxId_Choice, wxDefaultPosition, wxDefaultSize, StringTblChoice1and2);
	bSizer8->Add( m_ChoiceBand2, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer8, 1, 0, 5 );

	wxBoxSizer* bSizer9 = new wxBoxSizer( wxVERTICAL );

	m_staticText3 = new wxStaticText( this, wxID_ANY, _("3rd band (multiplier) :"));
	bSizer9->Add( m_staticText3, 0, wxALL, 5 );

	m_ChoiceBand3 = new wxChoice( this, wxId_Choice, wxDefaultPosition, wxDefaultSize, StringTblChoice3);
	bSizer9->Add( m_ChoiceBand3, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer9, 1, 0, 5 );

	wxBoxSizer* bSizer10;
	bSizer10 = new wxBoxSizer( wxVERTICAL );

	m_staticText4 = new wxStaticText( this, wxID_ANY, _("4th band (tolerance) :"));
	bSizer10->Add( m_staticText4, 0, wxALL, 5 );

	m_ChoiceBand4 = new wxChoice( this, wxId_Choice, wxDefaultPosition, wxDefaultSize, StringTblChoice4);
	bSizer10->Add( m_ChoiceBand4, 0, wxALL|wxEXPAND, 5 );

	SizerChoices->Add( bSizer10, 1, 0, 5 );

	MainSizer->Add( SizerChoices, 0, wxEXPAND, 5 );

    wxString ImgFileName = findFile(_T("data/resistance.png"));
    if(wxFileExists(ImgFileName))
    {
        ImgResistance.LoadFile(ImgFileName, wxBITMAP_TYPE_PNG);
        m_ImgResistance = new wxStaticBitmap( this, wxID_ANY, wxBitmap(ImgResistance));
    }
    else
    {
        wxMessageBox(_("Can't find resistance image. Will now exit.\nPath : ") + ImgFileName,_("Error, file not found"), wxICON_ERROR | wxOK);
        exit(EXIT_FAILURE);
    }

	MainSizer->Add( m_ImgResistance, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

	this->SetSizer( MainSizer );

    wxCommandEvent event;
	OnChoice(event);
}
